#include "\lw\include\gpib.h"
#include "\lw\include\formatio.h"
#include "sn488.h"

/*======= KEPCO SN488 ====================================================*/
/* LabWindows 2.0 Instrument Driver                                        */
/* Original Release: 6/92                                                  */
/* By: Peter Blanco , Kepco Inc.                                           */
/* Originally written in C                                                 */
/* Modification History: None                                              */
/*=========================================================================*/

static char cmd[50];
static char *hex_str;
static int init;
static double SN488_vmax;
static double SN488_imax;
static int SN488_type;
static int SN488_vrange;
static int SN488_crange;
static int SN488_addr;
static int SN488_err;

int SN488_invalid_integer_range();
int SN488_invalid_real_range();
int SN488_device_closed();
int SN488_write_data();

int SN488_init (int addr)
{
    if(init!=1){
        SN488_vmax=10;
        SN488_imax=10;
        SN488_type=0;
        hex_str="0123456789ABCDEF";
        init=1;
    }

/*  default configuration 0-10v voltage , 0-1v current */

    if (SN488_invalid_integer_range (addr, 0, 30, -1) != 0)
        return SN488_err;

/*  If device has not been opened (SN488_addr = 0), open it. */

    if (SN488_addr <= 0)    {
        CloseInstrDevs("sn488");
        SN488_addr = OpenDev ("", "sn488");
        if (SN488_addr <= 0)    {
            SN488_err = 220;
            return SN488_err;
        }
    }

/*  Change the primary address of the device    */

    if (ibpad (SN488_addr, addr) < 0)   {
        SN488_err = 233;
        return SN488_err;
    }


    return SN488_err;
}

/*=========================================================================*/

int SN488_config(int vrange,int crange,int type,double volt,double curr)
{
    if (SN488_device_closed())
        return SN488_err;

    if (SN488_invalid_integer_range(vrange,0,1,-1) )
        return SN488_err;

    if (SN488_invalid_integer_range(crange,0,1,-2) )
        return SN488_err;

    if (SN488_invalid_integer_range(type,0,1,-3) )
        return SN488_err;

    if (SN488_invalid_real_range(volt,0.0,10000.0,-4) )
        return SN488_err;

    if (SN488_invalid_real_range(curr,0.0,1000.0,-5) )
        return SN488_err;

    SN488_vmax = volt;
    SN488_imax = curr;
    SN488_type = type;
    SN488_vrange = vrange;
    SN488_crange = crange;
    SN488_err = 0;
    return 0;
}

int SN488_set(int chn,double volt,double curr)
{
    int d1,d2,d3,vrange,crange;
    char vout[4],iout[4],dstr[4];

    if (SN488_device_closed())
        return SN488_err;

    if (SN488_invalid_integer_range(chn,0,2,-1) )
        return SN488_err;

    if (SN488_invalid_real_range(volt,-SN488_vmax,SN488_vmax,-2) )
        return SN488_err;

    if (SN488_invalid_real_range(curr,-SN488_imax,SN488_imax,-3) )
        return SN488_err;

    vrange=SN488_vrange;
    if(volt<0){
        switch (vrange){
            case 0:
            case 2:
                vrange+=1;
                break;
            case 1:
            case 3:
                vrange-=1;
                break;
        }
    }
    crange=SN488_crange;
    if(curr<0){
        switch (crange){
            case 0:
            case 2:
                crange+=1;
                break;
            case 1:
            case 3:
                crange-=1;
                break;
        }
    }
    volt=fabs(volt);
    curr=fabs(curr);
    if(SN488_type){
        d1=(volt/SN488_vmax)*999;
        d2=(curr/SN488_imax)*999;
        Fmt(vout,"%s<%i[w3p0]",d1);
        Fmt(iout,"%s<%i[w3p0]",d2);
     }

     else{
         volt=(volt/SN488_vmax)*4095;
         d1=volt/256;
         volt-=(d1*256);
         d2=volt/16;
         d3=volt-(d2*16);
         vout[0]=hex_str[d1];
         vout[1]=hex_str[d2];
         vout[2]=hex_str[d3];

         curr=(curr/SN488_imax)*4095;
         d1=curr/256;
         curr-=(d1*256);
         d2=curr/16;
         d3=curr-(d2*16);
         iout[0]=hex_str[d1];
         iout[1]=hex_str[d2];
         iout[2]=hex_str[d3];
    }
    vout[3]=0;
    iout[3]=0;
    switch (chn){
        case 0:
            Fmt(cmd,"%s<1%i%s2%i%s",vrange,vout,crange,iout);
            break;
        case 1:
            Fmt(cmd,"%s<2%i%s",crange,iout);
            break;
        case 2:
            Fmt(cmd,"%s<1%i%s",vrange,vout);
            break;
    }
    return SN488_write_data(cmd,10);
}
/*=========================================================================*/
/* This function removes the instrument from the GPIB device table and     */
/* sets SN488_addr = 0.                                                            */
/*=========================================================================*/
int SN488_close ()
{
/*  Check for device closed */

    if (SN488_device_closed())
        return SN488_err;

/*  Close the device    */

    if (CloseDev (SN488_addr) < 0)
        SN488_err = 221;
    else
        SN488_err = 0;

    SN488_addr = 0;
    return SN488_err;
}

/*= UTILITY ROUTINES ======================================================*/

/*=========================================================================*/
/* This function checks an integer to see if it lies between a minimum     */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal to 0 for     */
/* success and -1 for error.                                               */
/*=========================================================================*/
int SN488_invalid_integer_range (val, min, max, err_code)
int val;
int min;
int max;
int err_code;
{
  if ((val < min) || (val > max)) {
    SN488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks a real number to see if it lies between a minimum  */
/* and maximum value.  If the value is out of range, set the global error  */
/* variable to the value err_code.  The return value is equal to 0 for     */
/* success and -1 for error.                                               */
/*=========================================================================*/
int SN488_invalid_real_range (val, min, max, err_code)
double val;
double min;
double max;
int err_code;
{
  if ((val < min) || (val > max)) {
    SN488_err = err_code;
    return -1;
  }
  return 0;
}

/*=========================================================================*/
/* This function checks to see if the module has been initialized.  If the */
/* device has not been opened, a -1 is returned, 0 otherwise.              */
/*=========================================================================*/
int SN488_device_closed ()
{
    if (SN488_addr <= 0)  {
        SN488_err = 232;
        return -1;
    }
    return 0;
}

/*=========================================================================*/
/* This function writes a buffer of data to the GPIB interface.  The       */
/* return value is equal to the global error variable.                     */
/*=========================================================================*/
int SN488_write_data (buf, cnt)
char *buf;
int cnt;
{
    if (ibwrt(SN488_addr, buf, (long)cnt) <= 0)
        SN488_err = 230;
    else
        SN488_err = 0;

    return SN488_err;
}


